###Monte Carlo Simulation###

#Packages#
using DataFrames, Distributions, Random, Plots, CSV, ProgressBars, Dates,  LaTeXStrings, StatsPlots
date = today()

###Tuning parameters values###
R_monte = 1000
R = 5
Rep = 500

##Choose DGP 1 or 2##
DGP = 1

##Grid points of beta2 for Figure 1##
Grid_b2 = [-1:1:9;]; Grid_b3 = [-1:1:9;]
Grid_b2_n = length(Grid_b2); Grid_b3_n = length(Grid_b3)

###Parameters of models###
beta1 = 0.5; beta2 = 1.5; beta3 = 1.5
gamma0 = -0.5; gamma1 = 0.5; gamma2 = -1.0; gamma3 = 0.5

if DGP == 1
    alpha0 = 3.0
else
    alpha0 = 1.6
end


###Distributions###
nom = Normal(0,1)
bin = Binomial(1,0.5)
ex = Exponential(1)

###Inference Function###
function Q_func(N,b,y1,y0,x1,x2,x3,p_d,R,e,B,K,L) 
    Ng = 0
    for r in 1:R
        Ng = Ng + (2*r)^2*4*9
    end
    nom1 = Normal(mean(x1),std(x1))
    A = 0
    sigma1N = 0
    m1N = 0
    v = zeros(N)
    for i in 1:N
        f = 0
        for j in 1:N
            A = ((y1[i]>=y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])+b[3]*(x3[i]-x3[j])>0)*1
                + (y1[j]>y0[i] && b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])+b[3]*(x3[j]-x3[i])>0)*1)
            m1N = m1N + A
            f = f + A
        end
        v[i] = f/N
    end
    m1N = m1N/(N*(N-1))
    sigma1N = var(v)
    m2N = zeros(R,2*R,2*R,2,2,3,3) 
    sigma2N = zeros(R,2*R,2*R,2,2,3,3)
    HR = zeros(R)
    for r=1:R
        Q = 0
        for k=1:(2*r), l=1:(2*r), m=0:1, n=0:1, o=-1:1:1, p=-1:1:1
            A = 0
            H = 0
            v = zeros(N)
            for i in 1:N
                f = 0
                if quantile(nom1,(k-1)/(2*r))<x1[i]<=quantile(nom1,k/(2*r)) && x2[i]==m && x3[i]==o
                    for j in 1:N
                        if quantile(nom1,(l-1)/(2*r))<x1[j]<=quantile(nom1,l/(2*r)) && x2[j]==n && x3[j]==p
                            A = (-1/2 + (y1[i]>=y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])+b[3]*(x3[i]-x3[j])>0)*1
                                + (y1[j]>y0[i] && b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])+b[3]*(x3[j]-x3[i])>0)*1)
                            H = H + A
                            f = f + A
                        end
                    end
                end
                v[i] = f/N
            end
            H = H/(N*(N-1))
            m2N[r,k,l,m+1,n+1,o+2,p+2] = H
            sigma2 = var(v)
            sigma2N[r,k,l,m+1,n+1,o+2,p+2] = sigma2
            Q = Q + ((H^2)/(sigma2 + e*sigma1N))*(H<0)
        end
        HR[r] = Q/(((2*r)^2)*4*9*(r^2+100))
    end
    cmi = zeros(Rep)
    for rep in 1:Rep
        v = rand(nom,Ng)
        HRQ = zeros(R)
        c = 0
        for r in 1:R
            Q = 0
            for k=1:(2*r),l=1:(2*r),m=0:1,n=0:1,o=-1:1:1,p=-1:1:1
                c = c + 1
                H = (sqrt(sigma2N[r,k,l,m+1,n+1,o+2,p+2])*v[c]
                    + sqrt(sigma1N)*B*sqrt(0.8*log(N)/log(log(N)))*(sqrt(N)*m2N[r,k,l,m+1,n+1,o+2,p+2]/(sqrt(sigma2N[r,k,l,m+1,n+1,o+2,p+2] + 0.0001*sigma1N)*K*sqrt(((1-L*p_d^(1/3))^(2/5))*0.6*log(N)))>1)*1)    
                Q = Q + ((H^2)/(sigma2N[r,k,l,m+1,n+1,o+2,p+2] + e*sigma1N))*(H<0)*1  
            end
            HRQ[r] = Q/(((2*r)^2)*4*9*(r^2+100))
        end
        cmi[rep] = sum(HRQ)
    end
    return [N*sum(HR),quantile(cmi,0.950001)]
end




### Monte Carlo Experiment for Fig1###

function montecarlo_fig1(N)
    R=5; e=0.0001; B=1; K=1; L=1
    result = zeros(Grid_b3_n,Grid_b2_n)
    for j in ProgressBar(1:Grid_b2_n), k in ProgressBar(1:Grid_b3_n)
        monte = zeros(R_monte)
        Random.seed!(123)
        for rr=ProgressBar(1:R_monte)
            x1 = rand(nom,N).*2
            x2 = rand(bin,N)
            x3 = rand([-1:1:1;],N)
            u = log.(rand(ex,N))
            v = log.(rand(ex,N))
            w = log.(rand(ex,N))
            y = exp.((beta1*x1 .+ beta2*x2 .+ beta3*x3 .+ u .+ v))
            c = exp.((alpha0 .+ (gamma0 .+ gamma1*x1 .+ gamma2*x2 .+ gamma3*x3).*u .+ w))
            d = (y.<=c)*1
            p_d = 1-mean(d)
            y0 = d.*y .+ (ones(N).-d).*c
            y1 = d.*y .+ (ones(N).-d)*100
            C = Q_func(N,[1,Grid_b2[j],Grid_b3[k]],y1,y0,x1,x2,x3,p_d,R,e,B,K,L)
            monte[rr] = (C[1]>C[2])*1 
        end
        result[k,j] = mean(monte)
    end
    return result
end

##Computation##
@time result100 = montecarlo_fig1(100)
@time result250 = montecarlo_fig1(250)
@time result500 = montecarlo_fig1(500)

CSV.write(string("momte carlo_d=3_n=100_",DGP,"_",date,".csv"), DataFrame(result100))
CSV.write(string("momte carlo_d=3_n=250_",DGP,"_",date,".csv"), DataFrame(result250))
CSV.write(string("momte carlo_d=3_n=500_",DGP,"_",date,".csv"), DataFrame(result500))
heatmap(result100, xticks=([1:1:11;], [-1:1:9;]), yticks=([1:1:11;], [-1:1:9;]), xaxis = (L"\beta_{2}"), yaxis = (L"\beta_{3}"), clim=(0.0,1.0), color = cgrad(:thermal))
savefig(string("monte carlo_d=3_DGP",DGP,"_result100_",date,".png"))
heatmap(result250, xticks=([1:1:11;], [-1:1:9;]), yticks=([1:1:11;], [-1:1:9;]), xaxis = (L"\beta_{2}"), yaxis = (L"\beta_{3}"), clim=(0.0,1.0), color = cgrad(:thermal))
savefig(string("monte carlo_d=3_DGP",DGP,"_result250_",date,".png"))
heatmap(result500, xticks=([1:1:11;], [-1:1:9;]), yticks=([1:1:11;], [-1:1:9;]), xaxis = (L"\beta_{2}"), yaxis = (L"\beta_{3}"), clim=(0.0,1.0), color = cgrad(:thermal))
savefig(string("monte carlo_d=3_DGP",DGP,"_result500_",date,".png"))

### Monte Carlo Experiment for sensitivity analysis###

function montecarlo(N,b,R,e,B,K,L)
    monte = zeros(R_monte)
    Random.seed!(123)
    for rr=ProgressBar(1:R_monte)
        x1 = rand(nom,N).*2
        x2 = rand(bin,N)
        x3 = rand([-1:1:1;],N)
        u = log.(rand(ex,N))
        v = log.(rand(ex,N))
        w = log.(rand(ex,N))
        y = exp.((beta1*x1 .+ beta2*x2 .+ beta3*x3 .+ u .+ v))
        c = exp.((alpha0 .+ (gamma0 .+ gamma1*x1 .+ gamma2*x2 .+ gamma3*x3).*u .+ w))
        d = (y.<=c)*1
        p_d = 1-mean(d)
        y0 = d.*y .+ (ones(N).-d).*c
        y1 = d.*y .+ (ones(N).-d)*100
        C = Q_func(N,b,y1,y0,x1,x2,x3,p_d,R,e,B,K,L)
        monte[rr] = (C[1]>C[2])*1
    end
    return mean(monte)
end

##Computation##
@time result = [
[montecarlo(250,[1,3,3],5,0.0001,1,1,1),montecarlo(250,[1,3,0],5,0.0001,1,1,1),montecarlo(250,[1,0,3],5,0.0001,1,1,1),montecarlo(250,[1,0,0],5,0.0001,1,1,1)],
[montecarlo(250,[1,3,3],5,0.001,1,1,1),montecarlo(250,[1,3,0],5,0.001,1,1,1),montecarlo(250,[1,0,3],5,0.001,1,1,1),montecarlo(250,[1,0,0],5,0.001,1,1,1)],
[montecarlo(250,[1,3,3],5,0.00001,1,1,1),montecarlo(250,[1,3,0],5,0.00001,1,1,1),montecarlo(250,[1,0,3],5,0.00001,1,1,1),montecarlo(250,[1,0,0],5,0.00001,1,1,1)],
[montecarlo(250,[1,3,3],3,0.0001,1,1,1),montecarlo(250,[1,3,0],3,0.0001,1,1,1),montecarlo(250,[1,0,3],3,0.0001,1,1,1),montecarlo(250,[1,0,0],3,0.0001,1,1,1)],
[montecarlo(250,[1,3,3],7,0.0001,1,1,1),montecarlo(250,[1,3,0],7,0.0001,1,1,1),montecarlo(250,[1,0,3],7,0.0001,1,1,1),montecarlo(250,[1,0,0],7,0.0001,1,1,1)],
[montecarlo(100,[1,3,3],5,0.001,1,1,1),montecarlo(100,[1,3,0],5,0.001,1,1,1),montecarlo(100,[1,0,3],5,0.001,1,1,1),montecarlo(100,[1,0,0],5,0.001,1,1,1)],
[montecarlo(100,[1,3,3],5,0.0001,1,1,1),montecarlo(100,[1,3,0],5,0.0001,1,1,1),montecarlo(100,[1,0,3],5,0.0001,1,1,1),montecarlo(100,[1,0,0],5,0.0001,1,1,1)],
[montecarlo(100,[1,3,3],5,0.00001,1,1,1),montecarlo(100,[1,3,0],5,0.00001,1,1,1),montecarlo(100,[1,0,3],5,0.00001,1,1,1),montecarlo(100,[1,0,0],5,0.00001,1,1,1)],
[montecarlo(500,[1,3,3],5,0.001,1,1,1),montecarlo(500,[1,3,0],5,0.001,1,1,1),montecarlo(500,[1,0,3],5,0.001,1,1,1),montecarlo(500,[1,0,0],5,0.001,1,1,1)],
[montecarlo(500,[1,3,3],5,0.0001,1,1,1),montecarlo(500,[1,3,0],5,0.0001,1,1,1),montecarlo(500,[1,0,3],5,0.0001,1,1,1),montecarlo(500,[1,0,0],5,0.0001,1,1,1)],
[montecarlo(500,[1,3,3],5,0.00001,1,1,1),montecarlo(500,[1,3,0],5,0.00001,1,1,1),montecarlo(500,[1,0,3],5,0.00001,1,1,1),montecarlo(500,[1,0,0],5,0.00001,1,1,1)],
[montecarlo(1000,[1,3,3],5,0.001,1,1,1),montecarlo(1000,[1,3,0],5,0.001,1,1,1),montecarlo(1000,[1,0,3],5,0.001,1,1,1),montecarlo(1000,[1,0,0],5,0.001,1,1,1)],
[montecarlo(1000,[1,3,3],5,0.0001,1,1,1),montecarlo(1000,[1,3,0],5,0.0001,1,1,1),montecarlo(1000,[1,0,3],5,0.0001,1,1,1),montecarlo(1000,[1,0,0],5,0.0001,1,1,1)],
[montecarlo(1000,[1,3,3],5,0.00001,1,1,1),montecarlo(1000,[1,3,0],5,0.00001,1,1,1),montecarlo(1000,[1,0,3],5,0.00001,1,1,1),montecarlo(1000,[1,0,0],5,0.00001,1,1,1)],
[montecarlo(250,[1,3,3],5,0.0001,1/2,1,1),montecarlo(250,[1,3,0],5,0.0001,1/2,1,1),montecarlo(250,[1,0,3],5,0.0001,1/2,1,1),montecarlo(250,[1,0,0],5,0.0001,1/2,1,1)],
[montecarlo(250,[1,3,3],5,0.0001,2,1,1),montecarlo(250,[1,3,0],5,0.0001,2,1,1),montecarlo(250,[1,0,3],5,0.0001,2,1,1),montecarlo(250,[1,0,0],5,0.0001,2,1,1)],
[montecarlo(250,[1,3,3],5,0.0001,1,1/2,1),montecarlo(250,[1,3,0],5,0.0001,1,1/2,1),montecarlo(250,[1,0,3],5,0.0001,1,1/2,1),montecarlo(250,[1,0,0],5,0.0001,1,1/2,1)],
[montecarlo(250,[1,3,3],5,0.0001,1,2,1),montecarlo(250,[1,3,0],5,0.0001,1,2,1),montecarlo(250,[1,0,3],5,0.0001,1,2,1),montecarlo(250,[1,0,0],5,0.0001,1,2,1)],
[montecarlo(250,[1,3,3],5,0.0001,1/2,1/2,1),montecarlo(250,[1,3,0],5,0.0001,1/2,1/2,1),montecarlo(250,[1,0,3],5,0.0001,1/2,1/2,1),montecarlo(250,[1,0,0],5,0.0001,1/2,1/2,1)],
[montecarlo(250,[1,3,3],5,0.0001,2,2,1),montecarlo(250,[1,3,0],5,0.0001,2,2,1),montecarlo(250,[1,0,3],5,0.0001,2,2,1),montecarlo(250,[1,0,0],5,0.0001,2,2,1)],
#[montecarlo(250,[1,3,3],5,0.0001,1,1,0),montecarlo(250,[1,3,0],5,0.0001,1,1,0),montecarlo(250,[1,0,3],5,0.0001,1,1,0),montecarlo(250,[1,0,0],5,0.0001,1,1,0)]
]

#Output the result as csv file#
CSV.write(string("result_sensitivity_d=3_DGP",DGP,"_",today(),".csv"),DataFrame(result))







