###Monte Carlo Simulation for Figure D.1###

#Packages#
using DataFrames, Distributions, Random, ProgressBars, CSV, Dates, Plots, LaTeXStrings, StatsPlots
date = today()

##Choose DGP=1 or 2##
DGP = 2

###Preliminary Value##
R_monte = 1000
Rep = 500
Grid_b = [-1:0.5:9;]; Grid_t = [-1:0.5:2.5;]
Grid_b_n = length(Grid_b); Grid_t_n = length(Grid_t)

###Parameters###
beta1 = 0.5
beta2 = 1.5
if DGP==1
    alpha = 3
else
    alpha = 1.6
end
gamma0 = -0.5
gamma1 = 0.5
gamma2 = -1.0

y_base = 0.773 #This value is the median of y
yc = 1.64

###Distributions###
nom = Normal(0,1)
bin = Binomial(1,0.5)
ex = Exponential(1)


###Inference Function###
function Q_func(N,b,t,yc,y1,y0,x1,x2,y_base,p_d,R,e,B,K,L)
    Ng = 0
    for r in 1:R
        Ng = Ng + (2*r)^2*4
    end
    nom1 = Normal(mean(x1),std(x1))
    A = zeros(2)
    sigma1N = zeros(2)
    m1N = zeros(2)
    v = zeros(2,N)
    for i in 1:N
        f = zeros(2)
        for j in 1:N
            A[1] = ((y1[i]>=y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>0)*1
                + (y1[j]>y0[i] && b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])>0)*1)
            A[2] = ((y1[i] >= yc) - (y0[j] >= y_base))*(b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>t)
                + ((y1[j] >= yc) - (y0[i] >= y_base))*(b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])>t)
            m1N[1] = m1N[1] + A[1]; m1N[2] = m1N[2] + A[2]
            f[1] = f[1] + A[1]; f[2] = f[2] + A[2]
        end
        v[1,i] = f[1]/N; v[2,i] = f[2]/N
    end
    m1N[1] = m1N[1]/(N*(N-1)); m1N[2] = m1N[2]/(N*(N-1))
    sigma1N[1] = var(v[1,:]); sigma1N[2] = var(v[2,:])
    sigma2 = zeros(2)
    m2N = zeros(2,R,2*R,2*R,2,2) 
    sigma2N = zeros(2,R,2*R,2*R,2,2) 
    HR = zeros(R)
    for r in 1:R
        Q = 0
        for k=1:(2*r), l=1:(2*r), m=0:1, n=0:1
            A = zeros(2)
            H = zeros(2)
            v = zeros(2,N)
            for i in 1:N
                f = zeros(2)
                if quantile(nom1,(k-1)/(2*r)) < x1[i] <= quantile(nom1,k/(2*r)) && x2[i]==m
                    for j in 1:N
                        if quantile(nom1,(l-1)/(2*r)) < x1[j] <= quantile(nom1,l/(2*r)) && x2[j]==n
                            A[1] = (-1/2 + (y1[i]>=y0[j] && b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>0)*1
                                + (y1[j]>y0[i] && b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])>0)*1)
                            A[2] = ((y1[i] >= yc) - (y0[j] >= y_base))*(b[1]*(x1[i]-x1[j])+b[2]*(x2[i]-x2[j])>t)
                                + ((y1[j] >= yc) - (y0[i] >= y_base))*(b[1]*(x1[j]-x1[i])+b[2]*(x2[j]-x2[i])>t)
                            H[1] = H[1] + A[1]; H[2] = H[2] + A[2]
                            f[1] = f[1] + A[1]; f[2] = f[2] + A[2] 
                        end 
                    end 
                end
                v[1,i] = f[1]/N; v[2,i] = f[2]/N 
            end
            H[1] = H[1]/(N*(N-1)); H[2] = H[2]/(N*(N-1))
            m2N[1,r,k,l,m+1,n+1] = H[1]; m2N[2,r,k,l,m+1,n+1] = H[2] 
            sigma2[1] = var(v[1,:]); sigma2[2] = var(v[2,:])
            sigma2N[1,r,k,l,m+1,n+1] = sigma2[1]; sigma2N[2,r,k,l,m+1,n+1] = sigma2[2] 
            Q = Q + ((H[1]^2)/(sigma2[1] + e*sigma1N[1]))*(H[1]<0)*1 + ((H[2]^2)/(sigma2[2] + e*sigma1N[2]))*(H[2]<0)*1
        end
        HR[r] = Q/(((2*r)^2)*4*(r^2+100))
    end
    cmi = zeros(Rep)
    H = zeros(2)
    for rep in 1:Rep
        v1 = rand(nom,Ng); v2 = rand(nom,Ng)
        HRQ = zeros(R)
        c = 0
        for r in 1:R
            Q = 0
            for k=1:(2*r), l=1:(2*r), m=0:1, n=0:1
                c = c + 1
                H[1] = (sqrt(sigma2N[1,r,k,l,m+1,n+1])*v1[c]
                    + sqrt(sigma1N[1])*B*sqrt(0.8*log(N)/log(log(N)))*(sqrt(N)*m2N[1,r,k,l,m+1,n+1]/(sqrt(sigma2N[1,r,k,l,m+1,n+1] + 0.0001*sigma1N[1])*K*sqrt(((1-L*p_d^(1/3))^(2/5))*0.6*log(N)))>1)*1)  
                H[2] = (sqrt(sigma2N[2,r,k,l,m+1,n+1])*v2[c]
                    + sqrt(sigma1N[2])*B*sqrt(0.8*log(N)/log(log(N)))*(sqrt(N)*m2N[2,r,k,l,m+1,n+1]/(sqrt(sigma2N[2,r,k,l,m+1,n+1] + 0.0001*sigma1N[2])*K*sqrt(((1-L*p_d^(1/3))^(2/5))*0.6*log(N)))>1)*1)    
                Q = Q + ((H[1]^2)/(sigma2N[1,r,k,l,m+1,n+1] + e*sigma1N[1]))*(H[1]<0)*1 + ((H[2]^2)/(sigma2N[2,r,k,l,m+1,n+1] + 0.0001*sigma1N[2]))*(H[2]<0)*1
            end
            HRQ[r] = Q/(((2*r)^2)*4*(r^2+100))
        end
        cmi[rep] = sum(HRQ)
    end
    return [N*sum(HR),quantile(cmi,0.950001)]
end


### Monte Carlo Experiment ###

function montecarlo_fig1(N,alpha,yc)
    R=5; e=0.0001; B=1; K=1; L=1
    result = zeros(Grid_b_n,Grid_t_n)
    for i=ProgressBar(1:Grid_b_n)
        for j=ProgressBar(1:Grid_t_n)
            monte = zeros(R_monte)
            Random.seed!(123)
            for rr=1:R_monte
                x1 = rand(nom,N).*2
                x2 = rand(bin,N)
                u = log.(rand(ex,N))
                v = log.(rand(ex,N))
                w = log.(rand(ex,N))
                y = exp.((beta1*x1 .+ beta2*x2 .+ u .+ v))
                c = exp.((alpha .+ (gamma0 .+ gamma1*x1 .+ gamma2*x2).*u .+ w))
                d = (y.<=c)*1
                p_d = 1-mean(d)
                y0 = d.*y .+ (ones(N).-d).*c
                y1 = d.*y .+ (ones(N).-d)*100
                C = Q_func(N,[1,Grid_b[i]],Grid_t[j],yc,y1,y0,x1,x2,y_base,p_d,R,e,B,K,L)
                monte[rr] = (C[1]>C[2])*1
            end
            result[i,j] = mean(monte)
        end
    end
    return result
end


function montecarlo(N,b,t,yc,R,e,B,K,L)
    monte = zeros(R_monte)
    Random.seed!(123)
    for rr=ProgressBar(1:R_monte)
        x1 = rand(nom,N).*2
        x2 = rand(bin,N)
        u = log.(rand(ex,N))
        v = log.(rand(ex,N))
        w = log.(rand(ex,N))
        y = exp.((beta1*x1 .+ beta2*x2 .+ u .+ v))
        c = exp.((alpha .+ (gamma0 .+ gamma1*x1 .+ gamma2*x2).*u .+ w))
        d = (y.<=c)*1
        p_d = 1-mean(d)
        y0 = d.*y .+ (ones(N).-d).*c
        y1 = d.*y .+ (ones(N).-d)*100
        C = Q_func(N,b,t,yc,y1,y0,x1,x2,y_base,p_d,R,e,B,K,L)
        monte[rr] = (C[1]>C[2])*1
    end
    return mean(monte)
end


@time result100 = montecarlo_fig1(100,alpha,yc) 
@time result250 = montecarlo_fig1(250,alpha,yc)
@time result500 = montecarlo_fig1(500,alpha,yc)

CSV.write(string("momte carlos_joint inference_b and T_n=100_DGP",DGP,"_",date,".csv"), DataFrame(result100))
CSV.write(string("momte carlos_joint inference_b and T_n=250_DGP",DGP,"_",date,".csv"), DataFrame(result250))
CSV.write(string("momte carlos_joint inference_b and T_n=500_DGP",DGP,"_",date,".csv"), DataFrame(result500))
heatmap(result100,xticks=([1:1:15;], [-1.0,"",-0.5,"",0.0,"",0.5,"",1.0,"",1.5,"",2.0,"",2.5]), yticks=([1:1:21;], [-1.0,"",0.0,"",1.0,"",2.0,"",3.0,"",4.0,"",5.0,"",6.0,"",7.0,"",8.0,"",9.0]), xaxis = (L"T(y)"), yaxis = (L"\beta_{2}"), color = cgrad(:thermal,[0.,0.5,1.0]),clims=(0,1))
savefig(string("monte carlo_joint inference b and T_DGP",DGP,"_result100_",date,".png"))
heatmap(result250,xticks=([1:1:15;], [-1.0,"",-0.5,"",0.0,"",0.5,"",1.0,"",1.5,"",2.0,"",2.5]), yticks=([1:1:21;], [-1.0,"",0.0,"",1.0,"",2.0,"",3.0,"",4.0,"",5.0,"",6.0,"",7.0,"",8.0,"",9.0]), xaxis = (L"T(y)"), yaxis = (L"\beta_{2}"), color = cgrad(:thermal,[0.,0.5,1.0]),clims=(0,1))
savefig(string("monte carlo_joint inference b and T_DGP",DGP,"_result250_",date,".png"))
heatmap(result500,xticks=([1:1:15;], [-1.0,"",-0.5,"",0.0,"",0.5,"",1.0,"",1.5,"",2.0,"",2.5]), yticks=([1:1:21;], [-1.0,"",0.0,"",1.0,"",2.0,"",3.0,"",4.0,"",5.0,"",6.0,"",7.0,"",8.0,"",9.0]), xaxis = (L"T(y)"), yaxis = (L"\beta_{2}"), color = cgrad(:thermal,[0.,0.5,1.0]),clims=(0,1))
savefig(string("monte carlo_joint inference b and T_DGP",DGP,"_result500_",date,".png"))

## Monte Carlo Experiment for sensitivity analysis###

#Computation
@time result = [[montecarlo(250,[1,3],1.5,yc,5,0.0001,1,1,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,1,1,1), montecarlo(250,[1,3],0,yc,5,0.0001,1,1,1), montecarlo(250,[1,0],0,yc,5,0.0001,1,1,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.001,1,1,1), montecarlo(250,[1,0],1.5,yc,5,0.001,1,1,1), montecarlo(250,[1,3],0,yc,5,0.001,1,1,1), montecarlo(250,[1,0],0,yc,5,0.001,1,1,1)],    
[montecarlo(250,[1,3],1.5,yc,5,0.00001,1,1,1), montecarlo(250,[1,0],1.5,yc,5,0.00001,1,1,1), montecarlo(250,[1,3],0,yc,5,0.00001,1,1,1), montecarlo(250,[1,0],0,yc,5,0.00001,1,1,1)],
[montecarlo(250,[1,3],1.5,yc,3,0.0001,1,1,1), montecarlo(250,[1,0],1.5,yc,3,0.0001,1,1,1), montecarlo(250,[1,3],0,yc,3,0.0001,1,1,1), montecarlo(250,[1,0],0,yc,3,0.0001,1,1,1)],
[montecarlo(250,[1,3],1.5,yc,7,0.0001,1,1,1), montecarlo(250,[1,0],1.5,yc,7,0.0001,1,1,1), montecarlo(250,[1,3],0,yc,7,0.0001,1,1,1), montecarlo(250,[1,0],0,yc,7,0.0001,1,1,1)],
[montecarlo(100,[1,3],1.5,yc,5,0.001,1,1,1), montecarlo(100,[1,0],1.5,yc,5,0.001,1,1,1), montecarlo(100,[1,3],0,yc,5,0.001,1,1,1), montecarlo(100,[1,0],0,yc,5,0.001,1,1,1)],    
[montecarlo(100,[1,3],1.5,yc,5,0.0001,1,1,1), montecarlo(100,[1,0],1.5,yc,5,0.0001,1,1,1), montecarlo(100,[1,3],0,yc,5,0.0001,1,1,1), montecarlo(100,[1,0],0,yc,5,0.0001,1,1,1)],
[montecarlo(100,[1,3],1.5,yc,5,0.00001,1,1,1), montecarlo(100,[1,0],1.5,yc,5,0.00001,1,1,1), montecarlo(100,[1,3],0,yc,5,0.00001,1,1,1), montecarlo(100,[1,0],0,yc,5,0.00001,1,1,1)],
[montecarlo(500,[1,3],1.5,yc,5,0.001,1,1,1), montecarlo(500,[1,0],1.5,yc,5,0.001,1,1,1), montecarlo(500,[1,3],0,yc,5,0.001,1,1,1), montecarlo(500,[1,0],0,yc,5,0.001,1,1,1)],    
[montecarlo(500,[1,3],1.5,yc,5,0.0001,1,1,1), montecarlo(500,[1,0],1.5,yc,5,0.0001,1,1,1), montecarlo(500,[1,3],0,yc,5,0.0001,1,1,1), montecarlo(500,[1,0],0,yc,5,0.0001,1,1,1)],
[montecarlo(500,[1,3],1.5,yc,5,0.00001,1,1,1), montecarlo(500,[1,0],1.5,yc,5,0.00001,1,1,1), montecarlo(500,[1,3],0,yc,5,0.00001,1,1,1), montecarlo(500,[1,0],0,yc,5,0.00001,1,1,1)],
[montecarlo(1000,[1,3],1.5,yc,5,0.001,1,1,1), montecarlo(1000,[1,0],1.5,yc,5,0.001,1,1,1), montecarlo(1000,[1,3],0,yc,5,0.001,1,1,1), montecarlo(1000,[1,0],0,yc,5,0.001,1,1,1)],    
[montecarlo(1000,[1,3],1.5,yc,5,0.0001,1,1,1), montecarlo(1000,[1,0],1.5,yc,5,0.0001,1,1,1), montecarlo(1000,[1,3],0,yc,5,0.0001,1,1,1), montecarlo(1000,[1,0],0,yc,5,0.0001,1,1,1)],
[montecarlo(1000,[1,3],1.5,yc,5,0.00001,1,1,1), montecarlo(1000,[1,0],1.5,yc,5,0.00001,1,1,1), montecarlo(1000,[1,3],0,yc,5,0.00001,1,1,1), montecarlo(1000,[1,0],0,yc,5,0.00001,1,1,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,1/2,1/2,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,1/2,1/2,1), montecarlo(250,[1,3],0,yc,5,0.0001,1/2,1/2,1), montecarlo(250,[1,0],0,yc,5,0.0001,1/2,1/2,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,1/2,1,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,1/2,1,1), montecarlo(250,[1,3],0,yc,5,0.0001,1/2,1,1), montecarlo(250,[1,0],0,yc,5,0.0001,1/2,1,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,1,1/2,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,1,1/2,1), montecarlo(250,[1,3],0,yc,5,0.0001,1,1/2,1), montecarlo(250,[1,0],0,yc,5,0.0001,1,1/2,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,2,2,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,2,2,1), montecarlo(250,[1,3],0,yc,5,0.0001,2,2,1), montecarlo(250,[1,0],0,yc,5,0.0001,2,2,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,2,1,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,2,1,1), montecarlo(250,[1,3],0,yc,5,0.0001,2,1,1), montecarlo(250,[1,0],0,yc,5,0.0001,2,1,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,1,2,1), montecarlo(250,[1,0],1.5,yc,5,0.0001,1,2,1), montecarlo(250,[1,3],0,yc,5,0.0001,1,2,1), montecarlo(250,[1,0],0,yc,5,0.0001,1,2,1)],
[montecarlo(250,[1,3],1.5,yc,5,0.0001,1,1,0), montecarlo(250,[1,0],1.5,yc,5,0.0001,1,1,0), montecarlo(250,[1,3],0,yc,5,0.0001,1,1,0), montecarlo(250,[1,0],0,yc,5,0.0001,1,1,0)]
]

#Output the result as csv file#
CSV.write(string("result_joint inference b and T_sensitivity_DGP",DGP,"_",date,".csv"),DataFrame(result))


