####Numerical Examples for Table 1#####

##Packages##
using DataFrames,Distributions,RoundingIntegers,Random,Plots,StatsPlots,ProgressBars,CSV,LaTeXStrings,Dates
date = today()

##sample size###
N = 20000

##Parameters##
beta1 = 0.5
beta2 = 1.5
gamma0 = -0.5
gamma1 = 0.5
gamma2 = -1.0

##Random Variables##
nom = Normal(0, 1)
bin = Binomial(1, 0.5) 
ex = Exponential(1)
u = log.(rand(ex,N)); u_a = log.(rand(ex,N)); u_b = log.(rand(ex,N))
v = log.(rand(ex,N)); v_a = log.(rand(ex,N)); v_b = log.(rand(ex,N))
w = log.(rand(ex,N)); w_a = log.(rand(ex,N)); w_b = log.(rand(ex,N))
x1 = rand(nom,N)*2
x2 = rand([0,1],N)
y = exp.(x1.*beta1 .+ x2.*beta2 .+ u .+ v)
y_base = median(y)

##Transformation function##
function T0y(y)
    return (log(y) - log(y_base))/beta1
end

###CObjective function###
function H_func(alpha,b,xa,xb)
    y_a = exp.(xa[1].*beta1 .+ xa[2].*beta2 .+ u_a .+ v_a)
    c_a = exp.((alpha .+ (gamma0 .+ gamma1*xa[1] .+ gamma2*xa[2]).*u_a .+ w_a))    
    y_b = exp.(xb[1].*beta1 .+ xb[2].*beta2 .+ u_b .+ v_b)
    c_b = exp.((alpha .+ (gamma0 .+ gamma1*xb[1] .+ gamma2*xb[2]).*u_b .+ w_b))
    d_a = (y_a .< c_a)*1; d_b = (y_b .< c_b)*1
    y0_a = d_a.*y_a + (ones(N).-d_a).*c_a
    y1_a = d_a.*y_a + (ones(N).-d_a)*100
    y0_b = d_b.*y_b + (ones(N).-d_b).*c_b
    y1_b = d_b.*y_b + (ones(N).-d_b)*100
    
    object = 0
    if (xa[1]-xb[1])*b[1] + (xa[2]-xb[2])*b[2] > 0
        object = mean(sign.(y1_a .- y0_b))
    end
    return object
end


function check_func(alpha,b,x1_grid)
    x1_grid_n = length(x1_grid)
    B = zeros(4)
    for j=1:x1_grid_n, k=1:x1_grid_n
        B[1] = H_func(alpha,b,[x1_grid[j],0],[x1_grid[k],1])
        B[2] = H_func(alpha,b,[x1_grid[j],0],[x1_grid[k],0])
        B[3] = H_func(alpha,b,[x1_grid[j],1],[x1_grid[k],1])
        B[4] = H_func(alpha,b,[x1_grid[j],1],[x1_grid[k],0])
        if minimum(B)<0
            break
        end
    end
    return minimum(B)
end


##Lower Bound Function##

function L_bound_func(alpha,b_grid_low,p,x1_grid)
    A=-1; b_grid = b_grid_low
    while A<0 && b_grid<=3
        A = check_func(alpha,[1,b_grid],x1_grid)
        b_grid = b_grid + p
    end 
    return return b_grid-p
end


##Upper Bound Function##

function U_bound_func(alpha,b_grid_up,p,x1_grid)
    A=-1; b_grid = b_grid_up
    while A<0 && b_grid>=3
        A = check_func(alpha,[1,b_grid],x1_grid)
        b_grid = b_grid - p
    end 
    return return b_grid+p
end



b_matrix = zeros(3,6)
alpha_vec = [100,3,1.6]; x1_grid_vec = [[-2.5:0.5:2.5;],[-5:0.5:5;],[-5:0.25:5;]]
b2_grid_vec = [[1.5,4],[1.5,4.5],[1,5.5]]; p=0.01

for i=ProgressBar(1:3), j=ProgressBar(1:3)
    b_matrix[i,(2*(j-1)+1)] = L_bound_func(alpha_vec[i],b2_grid_vec[i][1],p,x1_grid_vec[j])
    b_matrix[i,2*j] = U_bound_func(alpha_vec[i],b2_grid_vec[i][2],p,x1_grid_vec[j])
    b2_grid_vec[i][1] = b_matrix[i,(2*(j-1)+1)]; b2_grid_vec[i][2] = b_matrix[i,2*j]; 
end

CSV.write(string("numerical example_v2_d=2","_",date,".csv"),  DataFrame(b_matrix))

##Check function##
function H_func(t,yc,alpha,b,xa,xb) 
    y_a = exp.(xa[1].*beta1 .+ xa[2].*beta2 .+ u_a .+ v_a)
    c_a = exp.((alpha .+ (gamma0 .+ gamma1*xa[1] .+ gamma2*xa[2]).*u_a .+ w_a))    
    y_b = exp.(xb[1].*beta1 .+ xb[2].*beta2 .+ u_b .+ v_b)
    c_b = exp.((alpha .+ (gamma0 .+ gamma1*xb[1] .+ gamma2*xb[2]).*u_b .+ w_b))
    d_a = (y_a .< c_a)*1; d_b = (y_b .< c_b)*1
    y0_a = d_a.*y_a + (ones(N).-d_a).*c_a
    y1_a = d_a.*y_a + (ones(N).-d_a)*100
    y0_b = d_b.*y_b + (ones(N).-d_b).*c_b
    y1_b = d_b.*y_b + (ones(N).-d_b)*100
    dyc_a = (y1_a.> yc)*1
    dyc_b = (y1_b.> yc)*1
    dy_base_a = (y0_a.> y_base)*1
    dy_base_b = (y0_b.> y_base)*1
    
    object = 0
    if (xa[1]-xb[1])*b[1] + (xa[2]-xb[2])*b[2] > t
        object = mean(dyc_a .- dy_base_b)       
    end
    return object
end

function check_func(t,y,alpha,b,x1_grid)
    x1_grid_n = length(x1_grid)
    B = zeros(4)
    x1_grid = x1_grid[randperm(x1_grid_n)]
    for j=1:x1_grid_n, k=1:x1_grid_n
        B[1] = H_func(t,y,alpha,b,[x1_grid[j],0],[x1_grid[k],1])
        B[2] = H_func(t,y,alpha,b,[x1_grid[j],0],[x1_grid[k],0])
        B[3] = H_func(t,y,alpha,b,[x1_grid[j],1],[x1_grid[k],1])
        B[4] = H_func(t,y,alpha,b,[x1_grid[j],1],[x1_grid[k],0])
        if minimum(B)<0
            break
        end
    end
    return minimum(B)
end
    


##Function to generate result##
function bound_func(yc,t_grid_low,p,alpha,b,x1_grid)
    A=-1; t_grid = t_grid_low
    while A<0
        A = check_func(t_grid,yc,alpha,b,x1_grid)
        t_grid = t_grid + p
    end 
    #return max(t_grid_low,t_grid-p)
    return max(t_grid_low,t_grid)
end


function result_func(alpha,b,yc_grid,p,x1_grid)
    yc_grid_n=length(yc_grid)
    result = [yc_grid zeros(yc_grid_n)]
    t_grid_low = -7
    for i=ProgressBar(1:yc_grid_n)
        result[i,2] = bound_func(yc_grid[i],t_grid_low,p,alpha,b,x1_grid)
        t_grid_low = result[i,2]
    end
    return result'
end

yc_grid = [0.1:0.1:10;]

function save_result(result,i,j)
    plot(yc_grid,result[2,:], ylim=(-6,6), fill = (6), xlabel=L"y", ylabel=L"T(y)", label=L"T_{B_I}")
    plot!([0.1:0.05:10;],T0y.([0.1:0.05:10;]),color="black",label=L"T_{0}(y)",legend = :bottomright)#legend = :outertopright
    plot!(xticks=(vcat([0.1],[1:1:10;]), [0.1,"1","2","3","4","5","6","7","8","9","10"]))
    plot!(yticks=([-6:1:6;]))
    savefig(string("numerical example_T func_DGP",i,"_Grid",j,"_",date,".png"))
    CSV.write(string("numerical example_T func_DGP",i,"_Grid",j,"_",date,".csv"), DataFrame(result))
end
    

alpha_vec = [100,3,1.6]; x1_grid_vec = [[-2.5:0.5:2.5;],[-5:0.5:5;],[-5:0.2:5;]]
for i=ProgressBar(1:3), j=ProgressBar(1:3)
    b1 = [1,3]; b2 = [1,b_matrix[i,2*(j-1)+1]]; b3 = [1, b_matrix[i,2*(j-1)+2]]
    result1 = result_func(alpha_vec[i],b1,yc_grid,0.01,x1_grid_vec[j])
    result2 = result_func(alpha_vec[i],b2,yc_grid,0.01,x1_grid_vec[j])
    result3 = result_func(alpha_vec[i],b3,yc_grid,0.01,x1_grid_vec[j])
    result = result1
    for i=1:size(result)[2]
        result[2,i] = min(result1[2,i],result2[2,i],result3[2,i]) 
    end
    save_result(result,i,j)
end
