####Numerical Examples for Figure E.1#####

##Packages##
using DataFrames, Distributions, RoundingIntegers, Random, StatsPlots, Plots, ProgressBars, CSV, Dates, LaTeXStrings
date = today()

##sample size###
N = 20000

##Parameters##
beta1 = 0.5
beta2 = 1.5
beta3 = 1.5
gamma0 = -0.5
gamma1 = 0.5
gamma2 = -1.0
gamma3 = 0.5

##Random Variables##
nom = Normal(0, 1)
bin = Binomial(1, 0.5) 
ex = Exponential(1)
u = log.(rand(ex,N)); u_a = log.(rand(ex,N)); u_b = log.(rand(ex,N))
v = log.(rand(ex,N)); v_a = log.(rand(ex,N)); v_b = log.(rand(ex,N))
w = log.(rand(ex,N)); w_a = log.(rand(ex,N)); w_b = log.(rand(ex,N))

###CObjective function###
function H_func(alpha,b,xa,xb)
    y_a = exp.(xa[1].*beta1 .+ xa[2].*beta2 .+ xa[3].*beta3  .+ u_a .+ v_a)
    c_a = exp.((alpha .+ (gamma0 .+ gamma1*xa[1] .+ gamma2*xa[2] .+ gamma3*xa[3] ).*u_a .+ w_a))    
    y_b = exp.(xb[1].*beta1 .+ xb[2].*beta2 .+ xb[3].*beta3 .+ u_b .+ v_b)
    c_b = exp.((alpha .+ (gamma0 .+ gamma1*xb[1] .+ gamma2*xb[2] .+ gamma3*xb[3] ).*u_b .+ w_b))
    d_a = (y_a .< c_a)*1; d_b = (y_b .< c_b)*1
    y1_a = d_a.*y_a + (ones(N).-d_a)*100
    y0_b = d_b.*y_b + (ones(N).-d_b).*c_b
    
    object = 0
    if (xa[1]-xb[1])*b[1] + (xa[2]-xb[2])*b[2] + (xa[3]-xb[3])*b[3] > 0
        object = mean(sign.(y1_a .- y0_b))
    end
    return object
end


function check_func(alpha,b,x1_grid,x3_grid)
    x1_grid_n = length(x1_grid); x3_grid_n = length(x3_grid)
    B = zeros(4)
    x1_grid = x1_grid[randperm(x1_grid_n)]; x3_grid = x3_grid[randperm(x3_grid_n)]
    for j=1:x1_grid_n, k=1:x1_grid_n, l=1:x3_grid_n, m=1:x3_grid_n
        B[1] = H_func(alpha,b,[x1_grid[j],0,x3_grid[l]],[x1_grid[k],1,x3_grid[m]])
        B[2] = H_func(alpha,b,[x1_grid[j],0,x3_grid[l]],[x1_grid[k],0,x3_grid[m]])
        B[3] = H_func(alpha,b,[x1_grid[j],1,x3_grid[l]],[x1_grid[k],1,x3_grid[m]])
        B[4] = H_func(alpha,b,[x1_grid[j],1,x3_grid[l]],[x1_grid[k],0,x3_grid[m]])
        if minimum(B)<0
            break
        end
    end
    return (minimum(B)>=0)*1
end

##Bound Function##

function BI_set(alpha,b2_grid,b3_grid,x1_grid,x3_grid)
    b2_grid_n = length(b2_grid); b3_grid_n = length(b3_grid)    
    object = ones(b3_grid_n,b2_grid_n)
    for i=ProgressBar(1:b2_grid_n)
        Random.seed!(123)
        J=0; j=1
        while J==0 && j <= b3_grid_n
            object[j,i] = check_func(alpha,[1,b2_grid[i],b3_grid[j]],x1_grid,x3_grid)
            J = object[j,i]
            j = j+1
        end
        K=0; k=b3_grid_n
        while K==0 && k>=j
            object[k,i] = check_func(alpha,[1,b2_grid[i],b3_grid[k]],x1_grid,x3_grid)
            K = object[k,i]
            k = k -1
        end
    end
    return object
end



##Grid interval of parameter space##
bgrid = 0.02

##Grids of coefficient parameters and support of covariates for computation##
alpha_vec = [300,3,1.6]; b2_grid = [1.75:bgrid:4.25;]; b3_grid = [2.25:bgrid:3.75;] 
x1_grid_vec = [[-2.5:0.5:2.5;],[-5:0.5:5;],[-5:0.25:5;]]; x3_grid = [-1:1:1;]


function save_result(plotmatrix,i,j)
    StatsPlots.heatmap(b2_grid,b3_grid,plotmatrix, color = [:white, :blue], legend=false)
    plot!(xaxis = (L"\beta_{2}"), xticks= [1.75:0.25:4.25;], yaxis = (L"\beta_{3}"), yticks= [2.25:0.25:3.75;])
    savefig(string("numerical example_d=3_Model",i,"_XGrid",j,"_bGrid",bgrid,"_N=",N,"_",date,".png"))
    CSV.write(string("numerical example_d=3_Model",i,"_XGrid",j,"_bGrid",bgrid,"_N=",N,"_",date,".csv"),  DataFrame(plotmatrix), writeheader=false)
end

for i=ProgressBar(1:3), j=ProgressBar(1:3)
    plotmatrix = BI_set(alpha_vec[i],b2_grid,b3_grid,x1_grid_vec[j],x3_grid)
    save_result(plotmatrix,i,j)
end











